/*
	BANK.C

	28-jun-88, Marc Savary, Ad Lib Inc.

	Timbre Bank Manager.

	Refer to PLAY.C for compilation instructions.
*/

#include <stdio.h>
#include <fcntl.h>

#include "bank.h"

#define MICROSOFT 1
#define TIMBRE_NAME_SIZE	9
#define TIMBRE_DEF_LEN		28
#define TIMBRE_DEF_SIZE	(TIMBRE_DEF_LEN * sizeof( int))
#define MAJOR_VERSION		1
#define MINOR_VERSION		0

#ifdef LATTICE
extern char * getmem();
extern long lseek();
#endif

#ifdef MICROSOFT
#include  <malloc.h>
#include  <string.h>
#include  <io.h>
#include  <stdio.h>
#include  <sys\types.h>
#include  <sys\stat.h>
#define  open(x,y)      open(x,y,S_IWRITE)
#define  rlsmem(x,y)    free(x)
#define  getmem(x)      malloc(x)
#define  setmem(x,y,z)  memset(x,z,y)
#define  movmem(x,y,z)  memmove(y,x,z)
#endif

FILE *bfile;

/*
	Open the timbre bank 'bankName'. If does not exist, and
	'creatFlag' is != 0, create it.

	Return a pointer to the descriptor, or NULL if error.
*/
BankPtr OpenBank( bankName, createFlag)
	char * bankName;
	{
	BankRec * bP;

	bP = (BankRec *) getmem( sizeof( BankRec));
	if( NULL == bP)  return NULL;

	bfile = fopen( bankName,"rb");
   if (!bfile) return NULL;

	bP->inMemFlag = 0;
	return bP;
	}	/* OpenBank() */


/*
	Close the already opened bank. If the bank has beed modified,
	write the modifications to file.
*/
int CloseBank( bankPtr)
	BankPtr bankPtr;
	{
	int len;

	fclose( bfile);
	if( bankPtr->inMemFlag) 
      {
		rlsmem( bankPtr->timbBank, len);
		}
	rlsmem( bankPtr, sizeof( BankRec));
	return 1;
	}	/* CloseBank() */



/*
	Load all the bank in memory, for fast access. Allocate required
	memory.
	return 0 if error.
*/
int LoadBank( bankPtr)
	BankPtr bankPtr;
	{
	TimFileBank tB, * tBPtr;
	int len, i;
	long pos;

	fread((char *) &tB, sizeof( TimFileBank),1,bfile);

	len = tB.nrTimbre * (TIMBRE_NAME_SIZE + TIMBRE_DEF_SIZE);
	len += sizeof( TimFileBank);

	tBPtr = ( TimFileBank *) getmem( len);
	if( tBPtr == NULL)       return 0;

	fseek(bfile , 0L, 0);
	fread((char *) tBPtr, len,1,bfile);
	bankPtr->timbBank = tBPtr;
	bankPtr->inMemFlag = 1;
	return 1;
	}	/* LoadBank() */


/*
	Return the timbre definition in 'timbreDefPtr'.
	If 'timbreName' is not an empty string, search for in the index of
	'bankPtr' and set 'timbreIndex' to the found position value.
	If 'timbreName' is an empty string, use 'timbreIndex' as position.
*/
int GetTimbre( timbreName, timbreIndex, timbreDefPtr, bankPtr)
	char * timbreName;			/* name of timbre def. to return */
	int * timbreIndex;			/* used if 'timbreName' == "" */
	int * timbreDefPtr;			/* array for save to timbre def. */
	BankPtr bankPtr;			/* timbre bank ref. */
	{
		return InMemGetTimbre( timbreName, timbreIndex, timbreDefPtr, bankPtr);
	}	/* GetTimbre() */



/*
	Do the work of 'GetTimbre()' when bank is in memory.
*/
static int InMemGetTimbre( timbreName, timbreIndex, timbreDefPtr, bankPtr)
	char * timbreName;
	int * timbreIndex;			/* in/out */
	int * timbreDefPtr;
	BankPtr bankPtr;
	{
	TimFileBank tB;
	char * defPtr, * iP;
	int i, nrTimbre;

	i = * timbreIndex;
	nrTimbre = bankPtr->timbBank->nrTimbre;
	iP = (char *)bankPtr->timbBank + sizeof( TimFileBank);
	if( *timbreName) {
		for( i = 0; i < nrTimbre; i++, iP += TIMBRE_NAME_SIZE)
			if( 0 == strcmp( timbreName, iP))
				break;
		}
	if( i >= nrTimbre)
		return 0;
	defPtr = (char *)bankPtr->timbBank + bankPtr->timbBank->offsetDef;
	movmem( defPtr + i * TIMBRE_DEF_SIZE, timbreDefPtr, TIMBRE_DEF_SIZE);
	*timbreIndex = i;
	return 1;
	}	/* InMemGetTimbre() */




